/*
 *  $Id: BuggyCode.cpp,v 1.2 2003/01/05 12:30:15 omalley Exp $
 *  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
 *
 *  Example program from "Programming Mac OS X: A Guide for
 *  UNIX Developers" by Kevin O'Malley, Manning Publishing
 *  2002.
 */
#include <cstdio>
#include <ctime>
#include <cstdlib>
#include <cstring>

#include "BuggyCode.h"

const int MAX_TIME_STR = 30;
const int MAX_STR = 10;

/*
 Get a command, perform an action, and return a reply string
 to the client.
 */
bool
Process(char *s, char *reply)
{
  if ( (s == NULL) || (reply == NULL) ) {    
    strcpy(reply, "unknown command");
    LogCommand("Process arg null");
    return true;
    
  }
  
  LogCommand(s);

  if (strcmp(s, "leak") == 0) {
    strcpy(reply, MakeMemoryLeak());
  }
  else if (strcmp(s, "over-beg") == 0) {
    strcpy(reply, "over-beg");
    OverwriteBegOfBuffer();
  }
  else if (strcmp(s, "over-end") == 0) {
    strcpy(reply, "over-end");
    OverwriteEndOfBuffer();
  }
  else if (strcmp(s, "over-free") == 0) {
    strcpy(reply, "over-beg");
    WriteToFree();
  }
  else {
    strcpy(reply, "unknown command");
  }
  return false;
}

/*
 Leak memory.  
 */
char *
MakeMemoryLeak()
{
  time_t t = time(NULL);
  char *pTimeStr = new char[MAX_TIME_STR];
  strncpy(pTimeStr, ctime(&t), MAX_TIME_STR-1);
  return pTimeStr;
}

/*
 Allocate memory, delete it, and write to the
 de-allocated memory.
 */
void
WriteToFree()
{
  char *buf = new char[10];
  strcpy(buf, "AAAAAAAAA");
  delete [] buf;

  // Opps!, memory already de-allocated.
  strcpy(buf, "AAAAAAAAA");
}

/*
 Overwrite the beginning of an allocated block.
 */
void
OverwriteBegOfBuffer()
{
  char *buf = new char[10];
  strcpy(buf, "AAAAAAAAA");
  strcpy((buf-50), "ZZZZZZZZZZ");
  delete [] buf;
}

/*
 Overwrite the end of an allocated block.
 */
void
OverwriteEndOfBuffer()
{
  char *buf = new char[10];
  strcpy(buf, "AAAAAAAAA");
  strcpy((buf+50), "ZZZZZZZZZZ");
  delete [] buf;
}

/*
 Logging function.
 */
void
LogCommand(char *s)
{
  printf("BuggyServer (%ld): %s\n", time(NULL), s);
  fflush(stdout);
}

