/*
 *  $Id: MySockets.cpp,v 1.2 2003/01/05 12:36:46 omalley Exp $
 *  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
 *
 *  Example program from "Programming Mac OS X: A Guide for
 *  UNIX Developers" by Kevin O'Malley, Manning Publishing
 *  2002.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cerrno>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <unistd.h>

#include "MySockets.h"

/*
 Write a message to the socket.  This function is based on code from
 Stevens's "W. R. Stevens, UNIX network programming, 2nd ed. Upper
 Saddle River, NJ: Prentice Hall PTR, 1998."
 */
int
WriteString(int fd, char *ptr, int nBytes)
{
  int   nLeft, nWritten;

  if (fd < 0) {
    return 0;
  }

  if (ptr == NULL) {
    return 0;
  }

  if ( (nBytes <= 0) || (nBytes > MAX_STRING_LEN) ) {
    return 0;
  }

  nLeft = nBytes + 1;

  while (nLeft > 0) {
    nWritten = write(fd, ptr, (unsigned int)nLeft);
    if (nWritten <= 0)
      return(nWritten);         

    nLeft -= nWritten;
    ptr += nWritten;
  }
  return(nBytes - nLeft);
}

/*
 Connect to the host/port.
 */
int
SetupConnection(const char *serverName, int port, int *sd)
{
  int err, enable;
  struct sockaddr_in addr;
  struct hostent *ptr;

  if (serverName == NULL)
    return -2;

  if (port < 0)
    return -2;

  // Create a TCP socket
  *sd = socket(AF_INET, SOCK_STREAM, 0);
  if (*sd < 0)
    return -2;

  // Initialize socket address.
  (void)memset((char *) &addr, 0, sizeof(struct sockaddr_in));
  addr.sin_family = AF_INET;
  addr.sin_port = htons((u_short) port);
  // Convert "servname" to IP address.
  ptr = gethostbyname(serverName);
  if (ptr == NULL) {
    return -2;
  }

  (void)memcpy(&addr.sin_addr,
               ptr->h_addr,
               (unsigned int)ptr->h_length);

  // Set socket to KEEPALIVE
  enable = 1;
  err = setsockopt(*sd, SOL_SOCKET, SO_KEEPALIVE,
                   (char *) &enable, sizeof(int));
  if (err)
    return -2;

  err = connect(*sd,
                (struct sockaddr *) &addr,
                (unsigned int)sizeof(addr));
  if (err < 0) {
    close(*sd);
    return -2;
  }
  return 0;
}
