/*
 *  $Id: SlowRead.cpp,v 1.2 2003/01/05 12:36:46 omalley Exp $
 *  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
 *
 *  Example program from "Programming Mac OS X: A Guide for
 *  UNIX Developers" by Kevin O'Malley, Manning Publishing
 *  2002.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cerrno>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <unistd.h>

#include "Consts.h"
#include "SlowRead.h"

/*
 Read a line from a descriptor.  Read the line one byte at a time,
 looking for the newline or NUL terminator.  We store the newline
 or NUL in the buffer, then follow it with a null.

 This function is based on code from
 Stevens's "W. R. Stevens, UNIX network programming, 2nd ed. Upper
 Saddle River, NJ: Prentice Hall PTR, 1998."
*/
int
ReadString(int fd, char *ptr, int maxlen)
{
  int   n, rc;
  char  c, *pBuf;

  if (fd < 0) {
    return 0;
  }

  if (ptr == NULL) {
    return 0;
  }

  if ( (maxlen <= 0) || (maxlen > MAX_STRING_LEN) ) {
    return 0;
  }

  pBuf = ptr;
  for (n=1; n<maxlen; n++) {
    if ( (rc = read(fd, &c, 1)) == 1) {
      *pBuf++ = c;

      if (c == '\0') {
        break;        
      }
    }
    else if (rc == 0) {
      if (n == 1) {
        // EOF, no data read
        printf("EOF, no data read\n");
        return 0;
      }
      else {
        // EOF, some data was read
        printf("EOF, some data was read\n");
        break;
      }
    }
    else {
      /*
       If we get an interrupted system call or a resource temporarily
       unavailable error read again.
       */
      if ( (errno == EINTR) || (errno == EAGAIN)) {
        continue;
      }

      printf("errno\n");
      return 0;
    }
  }	

  *pBuf = '\0';
  fflush(stdout);

  return strlen(ptr);
}
