/*
 *  $Id: CocoaWGetController.m,v 1.1.1.1 2002/11/09 15:44:02 omalley Exp $
 *  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
 *
 *  Example program from "Programming Mac OS X: A Guide for
 *  UNIX Developers" by Kevin O'Malley, Manning Publishing
 *  2002.
 */
#import "CocoaWGetController.h"

#import "MyTask.h"
#import "AppSupport.h"

NSString *WGET_CMD = @"/sw/bin/wget";

@implementation CocoaWGetController

- (id) init
{
  self = [super init];
  param = [[WGetParameters alloc] init];

	directory = NSHomeDirectory();
  return self;
}

- (void) dealloc
{
  [param release];
  [super dealloc];
}

- (void)awakeFromNib
{
}

- (IBAction)handleDownload:(id)sender
{
  NSMutableArray *args;
  MyTask *task;
  
  [self displayCmdLine:@"Downloading files, please wait..."];
  
  /*
   Update the model by getting the values from the controls
   and setting the model (param).
   */
  [limController getParameters:param];
  [downloadController getParameters:param];
  [retrievalController getParameters:param];
  [htmlFtpController getParameters:param];

  [param setValue:@"url":[url stringValue]];
  
  [self raiseSheet];

  /*
   At this point we have all the parameters from the interface
   stored in the data model (dictionary).  Next, we construct the 
   wget command line from the data model and run the wget task by sending
   a message to the task object.
  */
  args = [param getData];

  task = [[MyTask alloc] init];
  [task runTask:WGET_CMD theDirectory:directory theArgs:args getOutputFrom:1];

  [self closeSheet:sender];

  [AppSupport setStatusMsgWithDate:theStatus theMsg:[task output]];
  [self displayCmdLine:@"Download complete."];

  if ([task exitStatus] != 0)
    NSRunAlertPanel(@"Error getting files", @"wget returned an error.", @"OK", NULL, NULL );

  [task release];
}

- (IBAction)handleOpen:(id)sender
{
  [param loadData:[AppSupport getFilename:@"Open CocoaWGet Senerio File"]];

  [limController reset:param];
  [downloadController reset:param];
  [retrievalController reset:param];
  [htmlFtpController reset:param];
}

- (IBAction)handleReset:(id)sender
{
  [param initToDefaults];
  [self reset:sender];
}

- (IBAction)handleSave:(id)sender
{
  /*
   Update the model by getting the values from the controls
   and setting the model (param).
   */
  [limController getParameters:param];
  [downloadController getParameters:param];
  [retrievalController getParameters:param];
  [htmlFtpController getParameters:param];

  [param saveData:[AppSupport getSaveFile:@"Save CocoaWGet Senerio File"]];
}

- (IBAction)handleViewParams:(id)sender
{
  [self displayCmdLine:@""];
}

- (void)displayCmdLine:(NSString *)headerStr
{
  /*
   Update the model by getting the values from the controls
   and setting the model (param).
   */
  [limController getParameters:param];
  [downloadController getParameters:param];
  [retrievalController getParameters:param];
  [htmlFtpController getParameters:param];

  [param formatCommandLine];

  [AppSupport setStatusMsgWithDate:theStatus theMsg:headerStr];  
  [AppSupport setStatusMsgWithDate:theStatus theMsg:[param getCommandLine]];  
}

- (void)reset:(id)sender
{
  [limController reset:param];
  [downloadController reset:param];
  [retrievalController reset:param];
  [htmlFtpController reset:param];  
}

- (void)raiseSheet
{
  [NSApp beginSheet:downloadWindow
    modalForWindow:mainWindow
    modalDelegate:self
    didEndSelector:NULL
    contextInfo:nil];
}

- (void)closeSheet:(id)sender
{
  [downloadWindow orderOut:sender];  
  [NSApp endSheet:downloadWindow];  
}
@end
