/*
 *  $Id: CocoaWGetController_new.m,v 1.2 2003/01/05 12:44:22 omalley Exp $
 *  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
 *
 *  Example program from "Programming Mac OS X: A Guide for
 *  UNIX Developers" by Kevin O'Malley, Manning Publishing
 *  2002.
 */
#import "CocoaWGetController_new.h"

#import "MyTask.h"
#import "AppSupport.h"

NSString *WGET_CMD = @"/sw/bin/wget";

@implementation CocoaWGetController

- (id) init
{
  self = [super init];
  param = [[WGetParameters alloc] init];
  directory = NSHomeDirectory();
  retrievalInProgress = NO;
  
  return self;
}

- (void) dealloc
{
  [param release];
  [super dealloc];
}

- (void)awakeFromNib
{
}

- (IBAction)handleDownload:(id)sender
{
  NSMutableArray *args;
    
  /*
   Update the model by getting the values from the controls
   and setting the model (param).
   */
  [limController getParameters:param];
  [downloadController getParameters:param];
  [retrievalController getParameters:param];
  [htmlFtpController getParameters:param];

  [param setValue:@"url":[url stringValue]];
  args = [param getData];

  if (retrievalInProgress) {
    // This stops the task and calls our callback (-processFinished)
    [task stopProcess];
    // Release the memory for this wrapper object
    [task release];
    task=nil;
    return;
  }
  else {
    // If the task is still sitting around from the last run, release it
    if (task!=nil)
      [task release];
    // Let's allocate memory for and initialize a new TaskWrapper object, passing
    // in ourselves as the controller for this TaskWrapper object, the path
    // to the command-line tool, and the contents of the text field that
    // displays what the user wants to search on
    task = [[TaskWrapper alloc] initWithController:self arguments:[NSArray arrayWithObjects:WGET_CMD,@"--help",nil]];
    // kick off the process asynchronously
    // [task startProcess];
    [task startProcess:WGET_CMD theDirectory:directory theArgs: args];
  }

#if 0
  [self raiseSheet];

  /*
   At this point we have all the parameters from the interface
   stored in the data model (dictionary).  Next, we construct the 
   wget command line from the data model and run the wget task by sending
   a message to the task object.
  */
  args = [param getData];

  //task = [[MyTask alloc] init];
  //[task runTask1:WGET_CMD theDirectory:directory theArgs:args getOutputFrom:1];

  [self closeSheet:sender];

  //[AppSupport setStatusMsgWithDate:theStatus theMsg:[task output]];
  //[self displayCmdLine:@"Download complete."];

  //if ([task exitStatus] != 0)
   // NSRunAlertPanel(@"Error getting files", @"wget returned an error.", @"OK", NULL, NULL );

  //[task release];
#endif
}

- (IBAction)handleOpen:(id)sender
{
  [param loadData:[AppSupport getFilename:@"Open CocoaWGet Senerio File"]];

  [limController reset:param];
  [downloadController reset:param];
  [retrievalController reset:param];
  [htmlFtpController reset:param];
}

- (IBAction)handleReset:(id)sender
{
  [param initToDefaults];
  [self reset:sender];
}

- (IBAction)handleSave:(id)sender
{
  /*
   Update the model by getting the values from the controls
   and setting the model (param).
   */
  [limController getParameters:param];
  [downloadController getParameters:param];
  [retrievalController getParameters:param];
  [htmlFtpController getParameters:param];

  [param saveData:[AppSupport getSaveFile:@"Save CocoaWGet Senerio File"]];
}

- (IBAction)handleViewParams:(id)sender
{
  [self displayCmdLine:@""];
}

- (void)displayCmdLine:(NSString *)headerStr
{
  /*
   Update the model by getting the values from the controls
   and setting the model (param).
   */
  [limController getParameters:param];
  [downloadController getParameters:param];
  [retrievalController getParameters:param];
  [htmlFtpController getParameters:param];

  [param formatCommandLine];

  [AppSupport setStatusMsgWithDate:theStatus theMsg:headerStr];  
  [AppSupport setStatusMsgWithDate:theStatus theMsg:[param getCommandLine]];  
}

- (void)reset:(id)sender
{
  [limController reset:param];
  [downloadController reset:param];
  [retrievalController reset:param];
  [htmlFtpController reset:param];  
}

- (void)raiseSheet
{
  [NSApp beginSheet:downloadWindow
    modalForWindow:mainWindow
    modalDelegate:self
    didEndSelector:NULL
    contextInfo:nil];
}

- (void)closeSheet:(id)sender
{
  [downloadWindow orderOut:sender];  
  [NSApp endSheet:downloadWindow];  
}

// This callback is implemented as part of conforming to the ProcessController protocol.
// It will be called whenever there is output from the TaskWrapper.
- (void)appendOutput:(NSString *)output
{
  // add the string (a chunk of the results from locate) to the NSTextView's
  // backing store, in the form of an attributed string
  [AppSupport setStatusMsgWithDate:theStatus theMsg:output];  

  // setup a selector to be called the next time through the event loop to scroll
  // the view to the just pasted text.  We don't want to scroll right now,
  // because of a bug in Mac OS X version 10.1 that causes scrolling in the context
  // of a text storage update to starve the app of events
  [self performSelector:@selector(scrollToVisible:) withObject:nil afterDelay:0.0];
}

// This routine is called after adding new results to the text view's backing store.
// We now need to scroll the NSScrollView in which the NSTextView sits to the part
// that we just added at the end
- (void)scrollToVisible:(id)ignore {
  [AppSupport scrollStatus:theStatus];  
}

// A callback that gets called when a TaskWrapper is launched, allowing us to do any setup
// that is needed from the app side.  This method is implemented as a part of conforming
// to the ProcessController protocol.
- (void)processStarted
{
  retrievalInProgress = YES;
  // clear the results
  [AppSupport setStatusMsgWithDate:theStatus theMsg:@""];  
  // change the "Sleuth" button to say "Stop"
  [getButton setTitle:@"Stop"];
}

// A callback that gets called when a TaskWrapper is completed, allowing us to do any cleanup
// that is needed from the app side.  This method is implemented as a part of conforming
// to the ProcessController protocol.
- (void)processFinished
{
  retrievalInProgress = NO;
  // change the button's title back for the next search
  [getButton setTitle:@"Get"];
}

@end
