/*
 *  $Id: DownloadController.m,v 1.1.1.1 2002/11/09 15:44:02 omalley Exp $
 *  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
 *
 *  Example program from "Programming Mac OS X: A Guide for
 *  UNIX Developers" by Kevin O'Malley, Manning Publishing
 *  2002.
 */
#import "DownloadController.h"
#import "AppSupport.h"

@implementation DownloadController

- (IBAction)handleSetConcatTo:(id)sender
{
  [concatFilesTo setStringValue:[AppSupport getFilename:@"Concatenate to File"]];
}

- (IBAction)handleSetOutputDir:(id)sender
{
  [outputDir setStringValue:[AppSupport getDirectory:@"Output Directory"]];
}

- (IBAction)reset:(WGetParameters *)param
{
  [limitDownloadSizeTo setStringValue:[param getValue:@"--quota="]];
  [limitDownloadSizeType setStringValue:[param getValue:@"-Q-size"]];
  [outputDir setStringValue:[param getValue:@"--directory-prefix="]];
  [pauseBetweenRetrievals setStringValue:[param getValue:@"--wait="]];
  [removeNDirComponent setStringValue:[param getValue:@"--cut-dirs="]];
  [waitBetweenFailedRetrievals setStringValue:[param getValue:@"--waitretry="]];
  [concatFilesTo setStringValue:[param getValue:@"--output-document="]];
  
  if ([[param getValue:@"--timestamp"] isEqualToString:@"1"])
    [noFilesUnlessNewerThanLocal setState:NSOnState];
  else
    [noFilesUnlessNewerThanLocal setState:NSOffState];

  if ([[param getValue:@"--no-directories"] isEqualToString:@"1"])
      [noDirCreateOnDownload setState:NSOnState];
  else
    [noDirCreateOnDownload setState:NSOffState];

  if ([[param getValue:@"--no-host-directories"] isEqualToString:@"1"])
      [noHostnamePrefix setState:NSOnState];
  else
    [noHostnamePrefix setState:NSOffState];
    if ([[param getValue:@"--no-clobber"] isEqualToString:@"1"])
      [overwriteFiles setState:NSOnState];
  else
    [overwriteFiles setState:NSOffState];
    if ([[param getValue:@"--server-response"] isEqualToString:@"1"])
      [printServerResponse setState:NSOnState];
  else
    [printServerResponse setState:NSOffState];
    if ([[param getValue:@"--proxy="] isEqualToString:@"1"])
      [proxyOn setState:NSOnState];
  else
    [proxyOn setState:NSOffState];
    if ([[param getValue:@"--continue"] isEqualToString:@"1"])
      [resumeDownload setState:NSOnState];
  else
    [resumeDownload setState:NSOffState];
    if ([[param getValue:@"--spider"] isEqualToString:@"1"])
      [spider setState:NSOnState];
  else
    [spider setState:NSOffState];

  if ([[param getValue:@"Q-size"] isEqualToString:@"k"])
    [limitDownloadSizeType selectItemAtIndex:0];
  else
     [limitDownloadSizeType selectItemAtIndex:1];
   
  [nRetries selectItemAtIndex:[[param getValue:@"--tries="] intValue]];
  [pauseBetweenRetrievals selectItemAtIndex:[[param getValue:@"--wait="] intValue]];
  [removeNDirComponent selectItemAtIndex:[[param getValue:@"--cut-dirs="] intValue]];
  [waitBetweenFailedRetrievals selectItemAtIndex:[[param getValue:@"--waitretry="] intValue]];
}

- (void)getParameters:(WGetParameters *)param
{
  NSString *s;

  [param setValue:@"--quota=":[limitDownloadSizeTo stringValue]];
  [param setValue:@"Q-size":[limitDownloadSizeType titleOfSelectedItem]];
  [param setValue:@"--tries=":[nRetries titleOfSelectedItem]];
  [param setValue:@"--output-document=":[concatFilesTo stringValue]];

  [param setValue:@"--directory-prefix=":[outputDir stringValue]];
  [param setValue:@"--wait=":[pauseBetweenRetrievals titleOfSelectedItem]];
  [param setValue:@"--cut-dirs=":[removeNDirComponent titleOfSelectedItem]];
  [param setValue:@"--waitretry=":[waitBetweenFailedRetrievals titleOfSelectedItem]];
 
  [param setValue:@"--no-directories":[NSString stringWithFormat:@"%d", [noDirCreateOnDownload state]]];
  [param setValue:@"--no-host-directories":[NSString stringWithFormat:@"%d", [noHostnamePrefix state]]];
  [param setValue:@"--no-clobber":[NSString stringWithFormat:@"%d", [overwriteFiles state]]];
  [param setValue:@"--server-response":[NSString stringWithFormat:@"%d", [printServerResponse state]]];

  s = [NSString stringWithFormat:@"%d", [proxyOn state]];
  if ([s isEqualToString:@"1"])
    [param setValue:@"--proxy=":@"on"];
  else
    [param setValue:@"--proxy=":@"off"];

  [param setValue:@"--continue":[NSString stringWithFormat:@"%d", [resumeDownload state]]];
  [param setValue:@"--spider":[NSString stringWithFormat:@"%d", [spider state]]];
  [param setValue:@"--timestamp":[NSString stringWithFormat:@"%d", [noFilesUnlessNewerThanLocal state]]];
}

@end
