/*
 *  $Id: TaskWrapper.h,v 1.1.1.1 2002/11/09 15:44:02 omalley Exp $
 *  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
 *
 *  Example program from "Programming Mac OS X: A Guide for
 *  UNIX Developers" by Kevin O'Malley, Manning Publishing
 *  2002.
 */
#import <Foundation/Foundation.h>

@protocol TaskWrapperController

// Your controller's implementation of this method will be called when output arrives from the NSTask.
// Output will come from both stdout and stderr, per the TaskWrapper implementation.
- (void)appendOutput:(NSString *)output;

// This method is a callback which your controller can use to do other initialization when a process
// is launched.
- (void)processStarted;

// This method is a callback which your controller can use to do other cleanup when a process
// is halted.
- (void)processFinished;

@end

@interface TaskWrapper : NSObject {
    NSTask 			*task;
    id				<TaskWrapperController>controller;
    NSArray			*arguments;
}

// This is the designated initializer - pass in your controller and any task arguments.
// The first argument should be the path to the executable to launch with the NSTask.
- (id)initWithController:(id <TaskWrapperController>)controller arguments:(NSArray *)args;

// This method launches the process, setting up asynchronous feedback notifications.
// - (void) startProcess;
- (void) stopProcess;

// This method stops the process, stoping asynchronous feedback notifications.
- (void)startProcess:(NSString *)taskName theDirectory:(NSString *)dir theArgs:(NSMutableArray *)args;
@end

