-- 
--  $Id: Application.applescript,v 1.3 2003/01/05 12:46:45 omalley Exp $ 
--  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
-- 
--  Example program from "Programming Mac OS X: A Guide for
--  UNIX Developers" by Kevin O'Malley, Manning Publishing
--  2002.
-- 

(* ==== Properties ==== *)

property processing : false
property DATA_FILE_MAC : ((path to startup disk as text) & "mem_log.dat")
property DATA_FILE_UNIX : "/mem_log.dat"
property PLOT_FORMAT_FILE_MAC : ((path to startup disk as text) & "mem_log.plt")
property PLOT_FORMAT_FILE_UNIX : "/mem_log.plt"
property PROCESSES_FILE_MAC : ((path to startup disk as text) & "mem_log.ps")
property PROCESSES_FILE_UNIX : "/mem_log.ps"

on clicked theObject
	tell window "MemoryTracker"
		set processName to contents of text field "process"
		set nSamples to contents of text field "samples"
		set nSleep to contents of text field "sleepsecs"
		set dataToPlotPos to contents of popup button "datatoplot"
		set dataToPlotName to title of current menu item of popup button "datatoplot"
	end tell
	
	if theObject is button "start" of window "MemoryTracker" then
		clickedStart(theObject, processName, nSamples, nSleep)
	else if theObject is button "view" of window "MemoryTracker" then
		clickedView()
	else if theObject is button "plot" of window "MemoryTracker" then
		clickedPlot(dataToPlotPos, dataToPlotName, processName, nSamples, nSleep)
		activate
	else if theObject is button "list" of window "MemoryTracker" then
		clickedList()
	end if
end clicked

(* Based on sample Apple code from AppleScript Studio Examples. *)
on clickedStart(theObject, processName, nSamples, nSleep)
	if title of theObject is "Start" then
		if not processing then
			if processName is equal to "" or nSamples is equal to "" or nSleep is equal to "" then
				ERROR_MSG("No process, nSamples, nSleep.")
				set title of theObject to "Start"
				return
			end if
			set processing to true
			runGrabMemory(processName, nSamples, nSleep)
			set processing to false
		end if
	end if
	
end clickedStart

on clickedView()
	tell application "TextEdit"
		activate
		open {(DATA_FILE_MAC) as alias}
	end tell
end clickedView

on clickedList()
	getAllProcesses()
	tell application "TextEdit"
		activate
		open {(PROCESSES_FILE_MAC) as alias}
	end tell
end clickedList

on clickedPlot(dataToPlotPos, dataToPlotName, processName, nSamples, nSleep)
	plot(dataToPlotPos, dataToPlotName, processName, nSamples, nSleep)
end clickedPlot

on runGrabMemory(processName, nSamples, nSleep)
	set fp to open for access file (DATA_FILE_MAC) with write permission
	set eof fp to 0
	close access fp
	
	set fp to open for access file (DATA_FILE_MAC) with write permission
	repeat with cnt from 1 to nSamples
		set msgPrefix to processName & ":" & nSamples & ":" & nSleep
		set memList to getMemory(processName)
		
		set theLine to ""
		set el to "processing sample: " & cnt & "/" & nSamples
		STATUS_MSG(el)
		repeat with n in memList
			set el to el & "."
			STATUS_MSG(el)
			set theLine to theLine & n & " "
		end repeat
		write theLine to fp starting at eof
		write return to fp starting at eof
		set el to "sleeping: " & nSleep & " (secs)"
		sleep(nSleep)
	end repeat
	close access fp
	STATUS_MSG("done processing")
	
end runGrabMemory


on dialog ended theObject with reply withReply
	(*Add your script here.*)
end dialog ended

on became main theObject
end became main

on plot(pos, theName, processName, nSamples, nSleep)
	set pos to pos + 1
	set msg to "calling gnuplot to plot " & theName
	set plotCmd to "plot " & "\"" & DATA_FILE_UNIX & "\" using " & pos & " with lines"
	set s to "plotting data for " & theName & ", process " & processName
	
	-- Clear the file.
	set fp to open for access file (PLOT_FORMAT_FILE_MAC) with write permission
	set eof fp to 0
	close access fp
	
	set fp to open for access file (PLOT_FORMAT_FILE_MAC) with write permission
	write "set xlabel " & "\"Samples\" " & return to fp
	set s to "\"" & theName & "\""
	write "set ylabel  " & s & return to fp
	
	
	set s to theName & ": (" & processName & "/" & nSamples & "/" & nSleep & ") - (Process/Samples/Delay)"
	write "set title  " & "\"" & s & "\"" & return to fp
	write "set yrange [ 0 : ]" & return to fp
	write "set grid" & return to fp
	write "set key right" & return to fp
	write "set data style linespoint" & return to fp
	write plotCmd & return to fp
	close access fp
	
	
	tell application "gnuplot-3.7.1d"
		activate
		-- exec "load \"/mem_log.plt\""
		set s to "load \"" & PLOT_FORMAT_FILE_UNIX & "\""
		exec s
	end tell
end plot

(* =============================================================== *)
(* ====================== Support Subroutines ========================== *)
(* =============================================================== *)

on ERROR_MSG(msg)
	tell window "MemoryTracker"
		set contents of text field "status" to "ERROR: " & msg
	end tell
end ERROR_MSG

on STATUS_MSG(msg)
	tell window "MemoryTracker"
		set contents of text field "status" to msg
	end tell
end STATUS_MSG

on getAllProcesses()
	set theResult to do shell script "ps xo ucomm"
	set fp to open for access file (PROCESSES_FILE_MAC) with write permission
	set eof fp to 0
	close access fp
	
	set fp to open for access file (PROCESSES_FILE_MAC) with write permission
	write theResult & return to fp
	close access fp
	set s to "processes written to file " & PROCESSES_FILE_MAC
	STATUS_MSG(s)
end getAllProcesses

on sleep(secs)
	set start_time to current date
	set delta_time to 0
	repeat until delta_time > secs
		set delta_time to (current date) - start_time
	end repeat
	return delta_time
end sleep

on getMemory(cmdToken)
	-- The code printed in the book includes awk to print out values.
	-- Using awk is not necessary and has been removed from this code.
	set psCmd to "ps axo %mem,%cpu,cpu,rss,rsz,time,vsz,ucomm | grep " & cmdToken & " | grep -v grep "
	set theResult to do shell script psCmd
	if theResult is equal to "" then
		STATUS_MSG("returned empty string")
		set memList to {}
		return memList
	end if
	
	-- The code printed in the book is incorrect, this code fixes the problem.
	set mem to 1st word of theResult
	set pCpu to 2nd word of theResult
	set cpu to 3rd word of theResult
	set rss to 4th word of theResult
	set RSZ to 5th word of theResult
	
	-- Make sure we get the right time
	set t to 6th word of theResult
	set t to t & ":" & 7th word of theResult
	
	-- Here we skip to the 8th word because time, written as 0:00.00,
	-- is read as two words - 0 and 00.00
	set VSZ to 8th word of theResult
	
	set memList to {mem, pCpu, cpu, rss, RSZ, t, VSZ}
	return memList
end getMemory

