#!/usr/bin/perl -w
# 
#  $Id: pim.pl,v 1.2 2003/01/05 12:48:57 omalley Exp $ 
#  Copyright (c) 2002 Kevin O'Malley. All rights reserved.
# 
#  Example program from "Programming Mac OS X: A Guide for
#  UNIX Developers" by Kevin O'Malley, Manning Publishing
#  2002.
#

use Foundation;
use strict;

my $PREFS_FILE = "./pim.prefs";
my $PIM_PATH = "./";
my $TMP_TXT_FILE = "tmp.txt";
my $TMP_PS_FILE = "tmp.ps";

# Allocate the preferences dictionary and read the file.
# If read, the dictionary contains preferences in key/value pairs.
my $pref = NSMutableDictionary->dictionary();
readPrefs($PREFS_FILE);

for(;;) {
    system("clear");
    print "=======================\n";
    print "My PIM (Ctrl-C to edit)\n";
    print "=======================\n";
    print "0. Edit preferences file\n";
    print "1. Edit reminders \n";
    print "2. Edit contacts\n";
    print "3. Edit tasks\n";
    print "4. Show tasks\n";
    print "5. Generate calendar (ps)\n";
    print "6. View calendar (txt)\n";
    print "7. Print calendar\n";
    print "8. Show system cal\n";
    print "9. Show todays reminders\n";
    print "-------------------\n";
    print "-1. Edit word list\n";
    print "-2. Edit notebook\n";
    print "-3. Edit quotes\n";
    print "-4. View quotes\n";
    print "=======================\n";
    print "-> ";
    my $s = <STDIN>;
    chop($s);

    if ($s == 0) {
	system(getPrefVal("editor") . " " . $PREFS_FILE);
    }
    elsif ($s == 1) {
	system(getPrefVal("editor") . " ~/.reminders");
    }
    elsif ($s == 2) {
	my $file = getPrefVal("file-path") . getPrefVal("contacts-file");
	system(getPrefVal("editor") . " $file");
    }
    elsif ($s == 3) {
	my $file = getPrefVal("file-path") . getPrefVal("tasks-file");
	system(getPrefVal("editor") . " $file");
    }
    elsif ($s == 4) {
		system("clear");
		show_tasks();
    }
    elsif ($s == 5) {
	my $file = getPrefVal("file-path") . $TMP_PS_FILE;
	system("remind -p ~/.reminders | rem2ps -l -sd 18 > $file");
    }
    elsif ($s == 6) {
	my $file = getPrefVal("file-path") . $TMP_TXT_FILE;
	system("remind -c ~/.reminders > $file");
	system(getPrefVal("viewer") . " $file");
    }
    elsif ($s == 7) {
	my $file = getPrefVal("file-path") . $TMP_PS_FILE;
	system("lpr $file");
    }
    elsif ($s == 8) {
	system("cal");
    }
    elsif ($s == 9) {
	system("rem");
    }
    elsif ($s == -1) {
	my $file = getPrefVal("file-path") . getPrefVal("words-file");
	system(getPrefVal("editor") . " $file");
    }
    elsif ($s == -2) {
	my $file = getPrefVal("file-path") . getPrefVal("notebook-file");
	system(getPrefVal("editor") . " $file");
    }
    elsif ($s == -3) {
	my $file = getPrefVal("file-path") . getPrefVal("quotes-file");
	system(getPrefVal("editor") . " $file");
    }
    elsif ($s == -4) {
	my $file = getPrefVal("file-path") . getPrefVal("quotes-file");
	system(getPrefVal("editor") . " $file");
    }
    print "** press return to continue **\n";
    $s = <STDIN>;

}

sub show_tasks {
    
    my $file = getPrefVal("file-path") . getPrefVal("tasks-file");
    open(INPUT, "< $file")
	|| die "could not open file in $file: $!\n";

    my $tmpFile = getPrefVal("file-path") . getPrefVal("tasks-file") . ".tmp";
    open(OUTPUT, "> $tmpFile")
	or die "could not open file out $tmpFile: $!\n";

    my $cnt = 0;
    my @open_tasks = "";
    my @closed_tasks = "";
    my $status = "";
    my $item = "";
    my $description = "";
    while(<INPUT>) {
	my $line = $_;
	chop($line);
	my $token = substr($line, 0, 1);
	
	if ($token eq "-") {
	    $cnt++;
	}

	if ($cnt == 3) {
	    $cnt = 0;

	    $item .= "\n".$description;

	    if ($status eq "open") {
		push(@open_tasks, $item);
	    }
	    elsif ($status eq "closed") {
		push(@closed_tasks, $item);
	    }
	    else {
		print "error\n";
		exit;
	    }
	}
	
	if ($line ne "") {
	    my @chunk = split(/:/, $line);
	        
	    if ($chunk[0] eq "Item") {
		$item = trim($chunk[1]);
		$item .= ":";
	    }
	    elsif ($chunk[0] eq "Status") {
		$status = trim($chunk[1]);
		$item .= $chunk[1];
		$item .= ":";
	    }
	    elsif ($chunk[0] eq "Catagory") {
		$item .= $chunk[1];
		$item .= ":";
	    }
	}
    }

    my $t = time;
    my $s = localtime($t);

    print OUTPUT "----------\n";
    print OUTPUT "Task List: $s\n";

    print OUTPUT "\n";
    print OUTPUT "Open Tasks\n";
    print OUTPUT "----------\n";
    foreach $item (@open_tasks) {
	if ($item ne "") {
	    my @chunk = split(/:/, $item);
	    $chunk[1] = trim($chunk[1]);
	    print OUTPUT "  * $chunk[0] ($chunk[1])\n";
	}
    }
    print OUTPUT "\n";
    print OUTPUT "Closed Tasks\n";
    print OUTPUT "------------\n";
    foreach $item (@closed_tasks) {
	if ($item ne "") {
	    my @chunk = split(/:/, $item);
	    $chunk[1] = trim($chunk[1]);
	    print OUTPUT "  * $chunk[0] ($chunk[1])\n";
	}
    }
    
    print OUTPUT "----------\n";
    close INPUT;
    close OUTPUT;
    
    system(getPrefVal("viewer") . " " . $tmpFile);
}


# Log an error and exit; adds newline to message.
sub logExit {
    my ($msg) = @_;
    print "$msg\n";
    exit;
}

sub readPrefs {
    my ($fName) = @_;
    $pref = NSDictionary->dictionaryWithContentsOfFile_($fName);
    if (!defined($pref)) {
	logExit("preferences file not read: $PREFS_FILE");
	exit;
    }
}

sub writePrefs {
    my ($fName) = @_;
    $pref->writeToFile_atomically_($fName, 0);
}

sub getPrefVal {
    my ($key, $val) = @_;
    my $value = $pref->objectForKey_($key);
    if (!defined($value)) {
	logExit("no value for key: $key");
	exit;
    }
    
    my $s = sprintf("%s", $value->cString());
    return $s;
}

sub setPrefVal {
    my ($key, $val) = @_;
    $pref->setObject_forKey_($val, $key);
}


# From the PerlCookbook, 1st edition, p. 30.
sub trim {
    my @out = @_;
    for (@out) {
        s/^\s+//;
        s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}
